@@ -20,11 +20,13 @@ our @EXPORT_OK = qw(
                        format_shell_completion
                );
 
-our $VERSION = '0.09'; # VERSION
-our $DATE = '2014-06-27'; # DATE
+our $VERSION = '0.10'; # VERSION
+our $DATE = '2014-06-29'; # DATE
 
 our %SPEC;
 
+# all complete_* routines accept hash/named args, the other accept positional.
+
 $SPEC{complete_array} = {
     v => 1.1,
     summary => 'Complete from array',
@@ -188,7 +190,7 @@ sub complete_file {
 
     my $w = complete_array(array=>\@words);
 
-    mimic_shell_dir_completion(completion=>$w);
+    mimic_shell_dir_completion($w);
 }
 
 # TODO: complete_user (probably in a separate module)
@@ -203,6 +205,9 @@ $SPEC{mimic_shell_dir_completion} = {
     summary => 'Make completion of paths behave more like shell',
     description => <<'_',
 
+Note for users: normally you just need to use `format_shell_completion()` and
+need not know about this function.
+
 This function employs a trick to make directory/path completion work more like
 shell's own. In shell, when completing directory, the sole completion for `foo/`
 is `foo/`, the cursor doesn't automatically add a space (like the way it does
@@ -216,6 +221,7 @@ shell won't automatically add a space because there are now more than one
 completion possible (`foo/` and `foo/ `).
 
 _
+    args_as => 'array',
     args => {
         completion => {
             schema=>'array*',
@@ -229,8 +235,7 @@ _
     },
 };
 sub mimic_shell_dir_completion {
-    my %args  = @_;
-    my $c  = $args{completion};
+    my $c = shift;
     return $c unless @$c == 1 && $c->[0] =~ m!/\z!;
     [$c->[0], "$c->[0] "];
 }
@@ -240,6 +245,9 @@ $SPEC{break_cmdline_into_words} = {
     summary => 'Break command-line string into words',
     description => <<'_',
 
+Note to users: this is an internal function. Normally you only need to use
+`parse_shell_cmdline`.
+
 The first step of shell completion is to break the command-line string
 (e.g. from COMP_LINE in bash) into words.
 
@@ -255,6 +263,7 @@ By default, this routine splits by spaces and tabs and takes into account
 backslash and quoting. Unclosed quotes won't generate error.
 
 _
+    args_as => 'array',
     args => {
         cmdline => {
             schema => 'str*',
@@ -268,12 +277,11 @@ _
     },
 };
 sub break_cmdline_into_words {
-    my %args = @_;
-    my $str = $args{cmdline};
+    my $cmdline = shift;
 
     # BEGIN stolen from Parse::CommandLine, with some mods
-    $str =~ s/\A\s+//ms;
-    $str =~ s/\s+\z//ms;
+    $cmdline =~ s/\A\s+//ms;
+    $cmdline =~ s/\s+\z//ms;
 
     my @argv;
     my $buf;
@@ -281,7 +289,7 @@ sub break_cmdline_into_words {
     my $double_quoted;
     my $single_quoted;
 
-    for my $char (split //, $str) {
+    for my $char (split //, $cmdline) {
         if ($escaped) {
             $buf .= $char;
             $escaped = undef;
@@ -383,7 +391,7 @@ sub parse_shell_cmdline {
 
     my @left;
     if (length($left)) {
-        @left = @{ break_cmdline_into_words(cmdline=>$left) };
+        @left = @{ break_cmdline_into_words($left) };
         # shave off $0
         substr($left, 0, length($left[0])) = "";
         $left =~ s/^\s+//;
@@ -394,7 +402,7 @@ sub parse_shell_cmdline {
     if (length($right)) {
         # shave off the rest of the word at "cursor"
         $right =~ s/^\S+//;
-        @right = @{ break_cmdline_into_words(cmdline=>$right) }
+        @right = @{ break_cmdline_into_words($right) }
             if length($right);
     }
     $log->tracef("\@left=%s, \@right=%s", \@left, \@right);
@@ -424,6 +432,14 @@ Usually, like in bash, we just need to output the entries one line at a time,
 with some special characters in the entry escaped using backslashes so it's not
 interpreted by the shell.
 
+This function accepts a hash, not an array. You can put the result of
+`complete_*` function in the `completion` key of the hash. The other keys can be
+added for hints on how to format the completion reply more
+correctly/appropriately to the shell. Known hints: `type` (string, can be
+`filename`, `env`, or others; this helps the routine picks the appropriate
+escaping), `is_path` (bool, if set to true then `mimic_shell_dir_completion`
+logic is applied).
+
 _
     args_as => 'array',
     args => {
@@ -454,6 +470,7 @@ sub format_shell_completion {
 
     $shcomp //= {};
     my $comp = $shcomp->{completion} // [];
+    $comp = mimic_shell_dir_completion($comp) if $shcomp->{is_path};
     my $type = $shcomp->{type} // '';
 
     my @lines;
@@ -486,7 +503,7 @@ Complete::Util - Shell completion routines
 
 =head1 VERSION
 
-This document describes version 0.09 of Complete::Util (from Perl distribution Complete-Util), released on 2014-06-27.
+This document describes version 0.10 of Complete::Util (from Perl distribution Complete-Util), released on 2014-06-29.
 
 =head1 DESCRIPTION
 
@@ -494,16 +511,50 @@ This module provides routines for doing programmable shell tab completion.
 Currently this module is geared towards bash, but support for other shells might
 be added in the future (e.g. zsh, fish).
 
-The C<complete_*()> routines are used to complete from a specific data source or
-for a specific type. They are the lower-level functions.
+For more information about bash programmable completion, please consult the bash
+manual. Basically bash allows you to call an external program (in our case, a
+Perl script) for completion. When a user asks for a completion of a command by
+pressing Tab, bash will invoke your program with COMP_LINE and COMP_POINT
+environment variables which contain, respectively, raw command line string and
+cursor position. You'll need to parse the command line into "words", come up
+with a list of completion for the word at cursor position, and output the list
+as lines to STDOUT. This module provides helper routines for that.
+
+Say we're writing a utility called C<progless> which will invoke B<less> on a
+program located in PATH (in other words, show a program's source using less).
+You want to provide a completion so that when you press Tab, a list of programs
+on PATH will be provided. To do this, you can write a Perl program as follows:
+
+ # progless-completion
+ #!/usr/bin/perl
+ use Complete::Util qw(parse_shell_cmdline complete_program);
+ my $cmdline = parse_shell_cmdline();
+ my $res = complete_program(word => $cmdline->{words}[0]);
+ print format_shell_completion({completion=>$res});
+
+You'll need to put this program somewhere in your PATH and then install it via
+bash command:
+
+ % complete -C progless-complete progless
+
+then you'll be able to do:
+
+ % progless <Tab>
+ % progless deb<Tab>
+
+Also, take a look at L<Perinci::CmdLine>, a CLI framework that lets you do
+completion more easily.
 
 =head1 FUNCTIONS
 
 
-=head2 break_cmdline_into_words(%args) -> array
+=head2 break_cmdline_into_words(@args) -> array
 
 Break command-line string into words.
 
+Note to users: this is an internal function. Normally you only need to use
+C<parse_shell_cmdline>.
+
 The first step of shell completion is to break the command-line string
 (e.g. from COMP_LINE in bash) into words.
 
@@ -635,6 +686,14 @@ Usually, like in bash, we just need to output the entries one line at a time,
 with some special characters in the entry escaped using backslashes so it's not
 interpreted by the shell.
 
+This function accepts a hash, not an array. You can put the result of
+C<complete_*> function in the C<completion> key of the hash. The other keys can be
+added for hints on how to format the completion reply more
+correctly/appropriately to the shell. Known hints: C<type> (string, can be
+C<filename>, C<env>, or others; this helps the routine picks the appropriate
+escaping), C<is_path> (bool, if set to true then C<mimic_shell_dir_completion>
+logic is applied).
+
 Arguments ('*' denotes required arguments):
 
 =over 4
@@ -655,10 +714,13 @@ A hash containing list of completions and other metadata. For example:
 Return value:
 
 
-=head2 mimic_shell_dir_completion(%args) -> array
+=head2 mimic_shell_dir_completion(@args) -> array
 
 Make completion of paths behave more like shell.
 
+Note for users: normally you just need to use C<format_shell_completion()> and
+need not know about this function.
+
 This function employs a trick to make directory/path completion work more like
 shell's own. In shell, when completing directory, the sole completion for C<foo/>
 is C<foo/>, the cursor doesn't automatically add a space (like the way it does
@@ -710,6 +772,9 @@ Return value:
 
 =head1 DEVELOPER'S NOTES
 
+This is an internal note only, module users are not required to read this
+section.
+
 We want to future-proof the API so future features won't break the API (too
 hardly). Below are the various notes related to that.
 
@@ -737,13 +802,22 @@ but possibly for the entire command). If the user wants to accept the
 suggestion, she can press the Right arrow key. This can be supported later by a
 function e.g. C<shell_complete()> which accepts the command line string.
 
+=head1 SEE ALSO
+
+Programmable Completion section in Bash manual:
+L<https://www.gnu.org/software/bash/manual/html_node/Programmable-Completion.html>
+
+L<http://blogs.perl.org/users/steven_haryanto/2014/06/one-final-rant-about-programmable-completion-in-bash.html>
+
+L<Perinci::CmdLine>, a CLI framework that uses this module.
+
 =head1 HOMEPAGE
 
 Please visit the project's homepage at L<https://metacpan.org/release/Complete-Util>.
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/sharyanto/perl-Complete-Util>.
+Source repository is at L<https://github.com/sharyanto/perl-SHARYANTO-Complete-Util>.
 
 =head1 BUGS
 
